# Portions of this file are taken from
# https://github.com/what-studio/profiling/tree/0.1.1,
# the license for which can be found in the "licenses/profiling.txt" file
# in this repository/package.
from __future__ import absolute_import

import sys
import signal
from six import iteritems
from six.moves import _thread

from auklet.broker import MQTTClient
from auklet.utils import get_mac, setup_thread_excepthook, create_dir
from auklet.monitoring.logging import AukletLogging
from auklet.monitoring.processing import Client
from auklet.stats import MonitoringTree
from auklet.errors import AukletConfigurationError

__all__ = ['Monitoring']

except_hook_set = False


class Monitoring(AukletLogging):
    #: The frames sampler.  Usually it is an instance of :class:`profiling.
    #: sampling.samplers.Sampler`
    sampler = None
    tree = None
    client = None
    broker = None
    version = ""
    monitor = True
    samples_taken = 0
    timer = signal.ITIMER_PROF
    sig = signal.SIGPROF
    stopping = False
    stopped = False

    interval = 1e-3  # 1ms

    total_samples = 0

    emission_rate = 60000  # 60 seconds
    hour = 3600000  # 1 hour

    def __init__(self, api_key=None, app_id=None, release=None, version="",
                 base_url="https://api.auklet.io/", monitoring=True):
        if release is None:
            raise AukletConfigurationError(
                "Must include release in Monitoring Creation")
        global except_hook_set
        sys.excepthook = self.handle_exc
        if not except_hook_set:
            # ensure not attempting to set threading excepthook more than once
            setup_thread_excepthook()
            except_hook_set = True
        self.auklet_dir = create_dir()
        self.version = version
        self.app_id = app_id
        self.mac_hash = get_mac()
        self.client = Client(api_key, app_id, release, base_url,
                             self.mac_hash, self.version, self.auklet_dir)
        self.emission_rate = self.client.update_limits()
        self.tree = MonitoringTree(self.mac_hash)
        self.broker = MQTTClient(self.client)
        self.monitor = monitoring
        signal.signal(self.sig, self.sample)
        signal.siginterrupt(self.sig, False)
        super(Monitoring, self).__init__()

    def start(self):
        # Set a timer which fires a SIGALRM every interval seconds
        if self.monitor:
            signal.setitimer(self.timer, self.interval, self.interval)

    def stop(self):
        self.stopping = True
        self.wait_for_stop()

    def wait_for_stop(self):
        while not self.stopped:
            pass

    def sample(self, sig, current_frame):
        """Samples the given frame."""
        if self.stopping:
            signal.setitimer(self.timer, 0, 0)
            self.stopped = True
            return
        current_thread = _thread.get_ident()
        for thread_id, frame in iteritems(sys._current_frames()):
            if thread_id == current_thread:
                frame = current_frame
            frames = []
            while frame is not None:
                frames.append(frame)
                frame = frame.f_back
            self.tree.update_hash(frames)
        self.total_samples += 1
        self.samples_taken += 1
        self.process_periodic()

    def process_periodic(self):
        if self.total_samples % self.emission_rate == 0:
            self.broker.produce(
                self.tree.build_msgpack_tree(self.client))
            self.tree.clear_root()
            self.samples_taken = 0
        if self.total_samples % self.hour == 0:
            self.emission_rate = self.client.update_limits()
            self.client.check_date()

    def handle_exc(self, type, value, traceback):
        self.broker.produce(
            self.client.build_msgpack_event_data(
                type, traceback, self.tree), "event")
        sys.__excepthook__(type, value, traceback)

    def log(self, msg, data_type, level="INFO"):
        self.broker.produce(
            self.client.build_msgpack_log_data(msg, data_type, level), "event")



from __future__ import print_function

from mock import patch
import paho.mqtt.client as mqtt

from auklet.monitoring import Monitoring
from src.benchmark import base


def without_auklet():
    """
    Runs tests for a baseline number on how long the tests should take.
    """
    print("\n\nStarting benchmark tests without the Auklet Agent...")
    base.start(state="WithoutAuklet")


@patch('auklet.monitoring.processing.Client.update_limits')
@patch('auklet.broker.MQTTClient._get_certs')
def with_auklet_and_mqtt(get_certs_mock, update_limits_mock):
    """
    With a little help from mock, this test will push data that
    would normally go to the front end, to a MQTT container created locally.
    """
    print("\n\nStarting benchmark tests with the Auklet Agent and MQTT...")

    def _load_conf(self):
        self.brokers = "mqtt"
        self.port = 1883

    def create_producer(self):
        # Make non SSL producer for testing
        self.producer = mqtt.Client()
        self.producer.on_disconnect = self.on_disconnect
        self.producer.connect_async("mqtt", 1883)
        self.producer.loop_start()

    def _get_conf(self):
        return True

    def register_device(self):
        return True

    update_limits_mock.return_value = 10000
    get_certs_mock.return_value = True

    conf_patcher = patch('auklet.broker.MQTTClient._read_from_conf',
                         new=_load_conf)
    producer_patcher = patch('auklet.broker.MQTTClient.create_producer',
                             new=create_producer)
    get_conf_patcher = patch('auklet.broker.MQTTClient._get_conf',
                             new=_get_conf)
    register_device_patcher = patch(
        'auklet.monitoring.processing.Client._register_device',
        new=register_device)
    conf_patcher.start()
    producer_patcher.start()
    get_conf_patcher.start()
    register_device_patcher.start()

    auklet_monitoring = Monitoring("", "", "")
    auklet_monitoring.start()
    base.start(state="WithAukletMQTT")

    conf_patcher.stop()
    producer_patcher.stop()


def display_complete_results():
    """
    This function displays the final
    compared result.  Results are in table format.
    """
    with open('/tmp/benchmark_results') as file:
        my_list = tuple(tuple(map(str, line.split())) for line in file)

    without_auklet_run_time = with_auklet_mmqt_run_time = 0
    number_of_tests = int(len(my_list) / 2)

    try:
        print("\n\nTests comparison.")
        print(my_list[0][0].split('_')[0],
              my_list[number_of_tests][0].split('_')[0], sep='\t')
        print("seconds", "seconds", "name", sep='\t\t')

        for i in range(0, number_of_tests):
            without_auklet = round(float(my_list[i][2]), 6)
            with_auklet_mmqt = round(float(my_list[i+number_of_tests][2]), 6)

            print(without_auklet, with_auklet_mmqt, my_list[i][1], sep='\t\t')

        for i in range(0, number_of_tests):  # Calculates total runtime
            without_auklet_run_time = \
                without_auklet_run_time + float(my_list[i][2])

            with_auklet_mmqt_run_time = with_auklet_mmqt_run_time + \
                float(my_list[number_of_tests+i][2])

        print("---")
        print(str(round(without_auklet_run_time, 6)),
              str(round(with_auklet_mmqt_run_time, 6)),
              "Total Run Time", sep='\t\t')

    except IndexError:
        print("Tests did not fully complete.")


def main():
    without_auklet()
    with_auklet_and_mqtt()
    display_complete_results()


if __name__ == "__main__":
    main()



import sys
import unittest

from auklet.errors \
    import AukletException, AukletConnectionError, AukletConfigurationError


class TestAukletException(unittest.TestCase):
    def test_auklet_exception(self):
        if sys.version_info < (3,):
            self.assertEqual(
                str(AukletException(Exception)),
                "<type 'exceptions.Exception'>")
        else:
            self.assertEqual(
                str(AukletException(Exception)),
                "<class 'Exception'>")


class TestAukletConnectionError(unittest.TestCase):
    def test_auklet_connection_error(self):
        self.assertEqual(str(AukletConnectionError()), "")


class TestAukletConfigurationError(unittest.TestCase):
    def test_auklet_configuration_error(self):
        self.assertEqual(str(AukletConfigurationError()), "")


if __name__ == '__main__':
    unittest.main()



import os
import unittest
import requests

from mock import patch
from requests import HTTPError

from auklet.utils import *
from auklet.monitoring.processing import Client
from auklet.errors import AukletConfigurationError

try:
    # For Python 3.0 and later
    from urllib.error import HTTPError, URLError
    from urllib.request import Request, urlopen
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import urlopen, Request, HTTPError, URLError


class TestUtils(unittest.TestCase):
    def setUp(self):
        with patch("auklet.monitoring.processing.Client._register_device",
                   new=self.__register_device):
            self.client = Client(
                api_key="", app_id="",
                base_url="https://api-staging.auklet.io/",
                auklet_dir=".auklet",
            )

    def test_open_auklet_url(self):
        url = self.client.base_url + "private/devices/config/"

        with patch('auklet.utils.urlopen') as url_open:
            self.assertIsNotNone(open_auklet_url(url, self.client.apikey))

            url_open.side_effect = HTTPError(
                url=None, code=401, msg=None, hdrs=None, fp=None)
            self.assertRaises(AukletConfigurationError,
                              lambda: open_auklet_url(url, self.client.apikey))

            url_open.side_effect = HTTPError(
                url=None, code=None, msg=None, hdrs=None, fp=None)
            self.assertRaises(HTTPError,
                              lambda: open_auklet_url(url, self.client.apikey))

            url_open.side_effect = URLError("")
            self.assertIsNone(open_auklet_url(url, self.client.apikey))

    def test_post_auklet_url(self):
        with patch("auklet.utils.requests.post") as request_mock:
            request_mock.side_effect = requests.HTTPError(None)
            res = post_auklet_url("example.com", "apikey", {})
            self.assertIsNone(res)

    def test_create_file(self):
        files = ['.auklet/local.txt', '.auklet/limits',
                 '.auklet/usage', '.auklet/communication']
        for f in files:
            file = False
            if os.path.isfile(f):
                file = True
            self.assertTrue(file)

    def test_clear_file(self):
        file_name = "unit_test_temp"
        with open(file_name, "w") as unit_test_temp_file:
            unit_test_temp_file.write("data")
        clear_file(file_name)
        self.assertEqual(os.path.getsize(file_name), 0)
        os.remove(file_name)

    def test_build_url(self):
        extension = str("private/devices/config/")
        self.assertEqual(
            build_url(self.client.base_url, extension),
            self.client.base_url + extension)

    def test_get_mac(self):
        self.assertNotEqual(get_mac(), None)

    def test_create_auklet_dir(self):
        self.assertTrue(create_dir(".test_auklet"))

    def test_create_dir_temp(self):
        with patch("os.access") as os_access_mock:
            os_access_mock.return_value = False
            self.assertTrue(create_dir(".temp_auklet"))

    def test_get_abs_path(self):
        path = os.path.abspath(__file__)
        self.assertEqual(get_abs_path(path + "/.auklet"), path)

        with patch('os.path.abspath') as mock_abspath:
            mock_abspath.side_effect = IndexError
            self.assertEqual(get_abs_path(path), '')

    def test_get_device_ip(self):
        self.assertIsNotNone(get_device_ip())
        with patch('auklet.utils.Request') as _Request:
            _Request.side_effect = HTTPError(None, None, None, None, None)
            self.assertIsNone(get_device_ip())
            _Request.side_effect = URLError(None)
            self.assertIsNone(get_device_ip())
            _Request.side_effect = Exception
            self.assertIsNone(get_device_ip())

    def test_setup_thread_excepthook(self):
        print("\nDue to the nature of this test and of the function itself, "
              "the two Stack Traces, Exception and KeyboardInterrupt, "
              "must print in order to prove validity of test.")
        from threading import Thread
        self.assertIsNone(setup_thread_excepthook())
        thread_except = Thread(target=self.throw_exception)
        thread_keyboard_interrupt = \
            Thread(target=self.throw_keyboard_interrupt)
        thread_except.start()
        thread_keyboard_interrupt.start()
        os.system("sleep 2")

    def test_version_info(self):
        self.assertNotEqual(None, b('b'))
        self.assertNotEqual(None, u(b'u'))

    def throw_exception(self):
        raise Exception

    def throw_keyboard_interrupt(self):
        raise KeyboardInterrupt

    @staticmethod
    def __register_device(self):
        return True


if __name__ == '__main__':
    unittest.main()



import unittest
from mock import patch

from auklet.monitoring import Monitoring
from auklet.errors import AukletConfigurationError


class TestMonitoring(unittest.TestCase):
    def setUp(self):
        with patch('auklet.broker.MQTTClient._get_conf') as _get_conf:
            with patch("auklet.monitoring.processing.Client._register_device",
                       new=self.__register_device):
                with patch("os.path.isfile") as is_file_mock:
                    is_file_mock.return_value = False
                    _get_conf.side_effect = self.get_conf
                    self.monitoring = Monitoring(
                        api_key="",
                        app_id="",
                        release="",
                        base_url="https://api-staging.io",
                        monitoring=True)
                    self.monitoring.monitor = True

    def test_start(self):
        self.assertIsNone(self.monitoring.start())
        self.monitoring.stop()

    def test_initialize_without_release(self):
        self.assertRaises(
            AukletConfigurationError, Monitoring,
            api_key="", app_id="", base_url="https://api.auklet.io/")

    def test_stop(self):
        self.monitoring.start()
        self.monitoring.stop()
        self.assertTrue(self.monitoring.stopping)

    def test_sample(self):
        class CoCode:
            co_code = None
            co_firstlineno = None
            co_name = None
        class FBack:
            f_back = None
            f_code = CoCode()
        class Frame:
            f_back = FBack()
            f_code = CoCode()

        def update_hash(self, stack):
            global test_sample_stack  # used to tell if stack was created
            test_sample_stack = stack

        with patch('auklet.stats.MonitoringTree.update_hash', new=update_hash):
            self.monitoring.sample(None, current_frame=Frame())
            self.assertIsNotNone(test_sample_stack)
            self.monitoring.sample(None, current_frame=Frame())
            self.assertTrue(test_sample_stack)

    def test_process_periodic(self):
        def produce(self, data):
            global test_process_periodic_produce_data
            test_process_periodic_produce_data = True

        def check_date(self):
            global test_process_periodic_check_date
            test_process_periodic_check_date = True

        with patch('auklet.broker.MQTTClient.produce', new=produce):
            with patch('auklet.monitoring.processing.Client.check_date',
                       new=check_date):
                self.monitoring.process_periodic()
                self.assertTrue(test_process_periodic_produce_data)
                self.assertTrue(test_process_periodic_check_date)
                self.assertEqual(60000, self.monitoring.emission_rate)

    def test_handle_exc(self):
        with patch('auklet.broker.MQTTClient.produce') as _produce:
            with patch(
                    'auklet.monitoring.processing.'
                    'Client.build_msgpack_event_data') \
                    as _build_msgpack_event_data:
                with patch('sys.__excepthook__') as ___excepthook__:
                    ___excepthook__.side_effect = self.__excepthook__
                    _build_msgpack_event_data.return_value = True
                    _produce.side_effect = self.produce
                    self.monitoring.handle_exc(None, None, None)
                    self.assertTrue(test_handle_exc___excepthook___)

    def test_log(self):
        with patch('auklet.broker.MQTTClient.produce') as _produce:
            _produce.side_effect = self.produce
            self.monitoring.log("", "")
            self.assertIsNotNone(test_log_data)

    def build_msgpack_tree(self, app_id):
        print(app_id)

    def __register_device(self):
        return True

    @staticmethod
    def produce(data, data_type):
        global test_log_data
        test_log_data = data

    @staticmethod
    def get_conf():
        return True

    @staticmethod
    def __excepthook__(type, value, traceback):
        global test_handle_exc___excepthook___
        test_handle_exc___excepthook___ = True


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import, division, unicode_literals

import msgpack

import pprint
import inspect
from uuid import uuid4
from time import time
from auklet.utils import get_agent_version

try:
    import psutil
except ImportError:  # pragma: no cover
    # Some platforms that applications could be running on require specific
    # installation of psutil which we cannot configure currently
    psutil = None

__all__ = ['MonitoringTree', 'Event', 'SystemMetrics', 'Function']


class Function(object):
    __slots__ = ['samples', 'line_num', 'func_name', 'file_path',
                 'children', 'parent']

    def __init__(self, line_num, func_name, file_path="",
                 parent=None, samples=1):
        self.line_num = line_num
        self.func_name = func_name
        self.parent = parent
        self.children = []
        self.file_path = file_path
        self.samples = samples

    def __str__(self):
        pp = pprint.PrettyPrinter()
        return pp.pformat(dict(self))

    def __iter__(self):
        yield "functionName", self.func_name
        yield "nSamples", self.samples
        yield "lineNumber", self.line_num
        yield "filePath", self.file_path
        yield "callees", [dict(item) for item in self.children]

    def has_child(self, test_child):
        for child in self.children:
            if test_child.func_name == child.func_name \
                    and test_child.file_path == child.file_path:
                return child
        return False


class Event(object):
    __slots__ = ['trace', 'exc_type', 'line_num', 'abs_path']

    def __init__(self, exc_type, tb, tree, abs_path):
        self.exc_type = exc_type.__name__
        self.line_num = tb.tb_lineno
        self.abs_path = abs_path
        self._build_traceback(tb, tree)

    def __iter__(self):
        yield "stackTrace", self.trace
        yield "excType", self.exc_type

    def _convert_locals_to_string(self, local_vars):
        for key in local_vars:
            if type(local_vars[key]) != str and type(local_vars[key]) != int:
                local_vars[key] = str(local_vars[key])
        return local_vars

    def _build_traceback(self, trace, tree):
        tb = []
        while trace:
            frame = trace.tb_frame
            path = tree.get_filename(frame.f_code, frame)
            tb.append({"functionName": frame.f_code.co_name,
                       "filePath": path,
                       "lineNumber": frame.f_lineno,
                       "locals":
                           self._convert_locals_to_string(frame.f_locals)})
            trace = trace.tb_next
        self.trace = tb


class MonitoringTree(object):
    __slots__ = ['commit_hash', 'public_ip', 'mac_hash',
                 'abs_path', 'root_func']
    cached_filenames = {}

    def __init__(self, mac_hash=None, release=None):
        from auklet.utils import get_device_ip, get_abs_path
        self.commit_hash = release
        self.public_ip = get_device_ip()
        self.abs_path = get_abs_path('.auklet/version')
        self.mac_hash = mac_hash
        self.root_func = None

    def get_filename(self, code, frame):
        key = code.co_code
        file_name = self.cached_filenames.get(code.co_code, None)
        if file_name is None:
            try:
                file_name = inspect.getsourcefile(frame) or \
                            inspect.getfile(frame)
            except (TypeError, AttributeError):
                # These functions will fail if the frame is of a
                # built-in module, class or function
                return None
            self.cached_filenames[key] = file_name
        return file_name

    def _create_frame_func(self, frame, root=False, parent=None):
        if root:
            return Function(
                line_num=1,
                func_name="root",
                parent=None,
                file_path="",
                samples=1
            )

        file_path = self.get_filename(frame.f_code, frame)
        return Function(
            line_num=frame.f_code.co_firstlineno,
            func_name=frame.f_code.co_name,
            parent=parent,
            file_path=file_path
        )

    def _build_tree(self, new_stack):
        root_func = self._create_frame_func(None, True)
        parent_func = root_func
        for frame in reversed(new_stack):
            current_func = self._create_frame_func(
                frame, parent=parent_func)
            parent_func.children.append(current_func)
            parent_func = current_func
        return root_func

    def _update_sample_count(self, parent, new_parent):
        if not new_parent.children:
            return True
        new_child = new_parent.children[0]
        has_child = parent.has_child(new_child)
        if has_child:
            has_child.samples += 1
            return self._update_sample_count(has_child, new_child)
        parent.children.append(new_child)

    def update_hash(self, new_stack):
        new_tree_root = self._build_tree(new_stack)
        if self.root_func is None:
            self.root_func = new_tree_root
            return self.root_func
        self._update_sample_count(self.root_func, new_tree_root)

    def clear_root(self):
        self.root_func = None
        return True

    def build_tree(self, client):
        if self.root_func is not None:
            return {
                "application": client.app_id,
                "publicIP": self.public_ip,
                "id": str(uuid4()),
                "timestamp": int(round(time() * 1000)),
                "macAddressHash": self.mac_hash,
                "release": self.commit_hash,
                "agentVersion": get_agent_version(),
                "tree": dict(self.root_func),
                "device": client.broker_username,
                "absPath": client.abs_path,
                "version": client.version
            }
        return {}

    def build_msgpack_tree(self, client):
        return msgpack.packb(self.build_tree(client), use_bin_type=False)


class SystemMetrics(object):
    cpu_usage = 0.0
    mem_usage = 0.0
    inbound_network = 0
    outbound_network = 0
    prev_inbound = 0
    prev_outbound = 0

    def __init__(self):
        if psutil is not None:
            self.cpu_usage = psutil.cpu_percent(interval=1)
            self.mem_usage = psutil.virtual_memory().percent

    def __iter__(self):
        yield "cpuUsage", self.cpu_usage
        yield "memoryUsage", self.mem_usage
        yield "inboundNetwork", self.inbound_network
        yield "outboundNetwork", self.outbound_network



from __future__ import absolute_import

import os
import ssl
import json
import logging
import paho.mqtt.client as mqtt

from auklet.utils import build_url, create_file, open_auklet_url, u

try:
    # For Python 3.0 and later
    from urllib.error import HTTPError, URLError
    from urllib.request import Request, urlopen
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import urlopen, Request, HTTPError, URLError

__all__ = ["MQTTClient"]


class MQTTClient(object):
    producer = None
    brokers = None
    client = None
    username = None
    password = None
    com_config_filename = ".auklet/communication"
    port = None
    producer_types = {
        "monitoring": "python/profiler/{}/{}",
        "event": "python/events/{}/{}",
    }

    def __init__(self, client):
        self.client = client
        self._get_conf()
        self.create_producer()
        topic_suffix = "{}/{}".format(
            self.client.org_id, self.client.broker_username)
        self.producer_types = {
            "monitoring": "python/profiler/{}".format(topic_suffix),
            "event": "python/events/{}".format(topic_suffix),
        }

    def _write_conf(self, info):
        with open(self.com_config_filename, "w") as conf:
            conf.write(json.dumps(info))

    def _get_conf(self):
        res = open_auklet_url(
            build_url(
                self.client.base_url, "private/devices/config/"
            ),
            self.client.apikey
        )
        loaded = json.loads(u(res.read()))
        self._write_conf(loaded)
        self._read_from_conf(loaded)

    def _get_certs(self):
        if not os.path.isfile("{}/ca.pem".format(self.client.auklet_dir)):
            url = Request(
                build_url(self.client.base_url, "private/devices/certificates/"),
                headers={"Authorization": "JWT %s" % self.client.apikey})
            try:
                try:
                    res = urlopen(url)
                except HTTPError as e:
                    # Allow for accessing redirect w/o including the
                    # Authorization token.
                    res = urlopen(e.geturl())
            except URLError:
                return False
            filename = "{}/ca.pem".format(self.client.auklet_dir)
            create_file(filename)
            f = open(filename, "wb")
            f.write(res.read())
        return True

    def _read_from_conf(self, data):
        self.brokers = data['brokers']
        self.port = int(data['port'])

    def on_disconnect(self, client, userdata, rc):
        if rc != 0:
            logging.debug("Unexpected disconnection from MQTT")

    def create_producer(self):
        if self._get_certs():
            self.producer = mqtt.Client(client_id=self.client.client_id,
                                        protocol=mqtt.MQTTv311,
                                        transport="ssl",
                                        clean_session=False)
            self.producer.username_pw_set(
                username=self.client.broker_username,
                password=self.client.broker_password)
            self.producer.enable_logger()
            context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
            context.verify_mode = ssl.CERT_REQUIRED
            context.load_verify_locations(
                capath="{}/".format(self.client.auklet_dir))
            context.options &= ~ssl.OP_NO_SSLv3
            self.producer.tls_set_context()
            self.producer.on_disconnect = self.on_disconnect
            self.producer.connect_async(self.brokers, self.port)
            self.producer.loop_start()

    def produce(self, data, data_type="monitoring"):
        self.producer.publish(
            self.producer_types[data_type], payload=data, qos=1)



import sys
import unittest
from mock import patch

from auklet.stats import Function, Event, MonitoringTree, SystemMetrics

try:
    import psutil
except ImportError:
    # Some platforms that applications could be running on require specific
    # installation of psutil which we cannot configure currently
    psutil = None


class TestFunction(unittest.TestCase):
    def setUp(self):
        self.function = Function(line_num=0, func_name="UnitTest")

    def test___str__(self):
        self.assertNotEqual(self.function.__str__(), None)

    def test___iter__(self):
        for t in self.function.__iter__():
            self.assertNotEqual(t, None)

    def test_has_child(self):
        self.assertFalse(self.function.has_child(test_child=""))
        self.function.children.append(self.get_test_child())
        self.assertNotEqual(
            self.function.has_child(test_child=self.get_test_child()), False)

    def get_test_child(self):
        class TestChild:
            func_name = "test_child"
            file_path = "file_path"
        return TestChild


class TestEvent(unittest.TestCase):
    def setUp(self):
        self.tree = MonitoringTree()
        self.patcher = patch(
            'auklet.stats.MonitoringTree.get_filename', new=self.get_filename)
        self.patcher.start()
        self.event = Event(
            exc_type=str, tb=self.get_traceback(),
            tree=self.tree, abs_path="/")

    def tearDown(self):
        self.patcher.stop()

    def test___iter__(self):
        for t in self.event.__iter__():
            self.assertNotEqual(t, None)

    def test_convert_locals_to_string(self):
        self.assertNotEqual(
            self.event._convert_locals_to_string(
                local_vars={"key": "value"}), None)
        self.assertNotEqual(
            self.event._convert_locals_to_string(
                local_vars={"key": True}), None)

    def test_build_traceback(self):
        self.build_patcher('auklet.stats.MonitoringTree.get_filename', "")

    def get_filename(self, code="code", frame="frame"):
        _ = code
        _ = frame
        return ""

    def get_traceback(self):
        class Code:
            co_name = None
        class Frame:
            f_code = Code()
            f_lineno = 0
            f_locals = {"key": "value"}
        class Traceback:
            tb_lineno = 0
            tb_frame = Frame()
            tb_next = None
        return Traceback

    def build_patcher(self, patch_location, return_value,
                      expected=[{'functionName': None,
                                 'filePath': '',
                                 'lineNumber': 0,
                                 'locals': {'key': 'value'}}]):

        with patch(patch_location) as mock_patcher:
            mock_patcher.return_value = return_value
            self.event._build_traceback(
                trace=self.get_traceback(), tree=self.tree)
            self.assertEqual(expected, self.event.trace)



class TestMonitoringTree(unittest.TestCase):
    def setUp(self):
        self.monitoring_tree = MonitoringTree()
        self.function = Function(line_num=0, func_name="UnitTest")

    def test_get_filename(self):
        self.monitoring_tree.cached_filenames.clear()
        self.monitoring_tree.cached_filenames['file_name'] = "file_name"
        self.assertEqual(
            self.monitoring_tree.get_filename(
                code=self.get_code(), frame="frame"), self.get_code().co_code)

        self.monitoring_tree.cached_filenames.clear()
        self.monitoring_tree.cached_filenames['file_name'] = None
        self.assertIsNone(self.monitoring_tree.get_filename(
            code=self.get_code(), frame="frame"))

        with patch('inspect.getsourcefile') as _get_source_file:
            _get_source_file.return_value = "file_name"
            self.assertIsNotNone(self.monitoring_tree.get_filename(
                code=self.get_code(), frame="frame"))

    def test_create_frame_function(self):
        class Code:
            co_firstlineno = 0
            co_name = ""

        class Frame:
            f_code = Code

        self.assertIsNotNone(
            self.monitoring_tree._create_frame_func("", root=True))
        with patch(
                'auklet.stats.MonitoringTree.get_filename') as _get_filename:
            _get_filename.return_value = None
            self.assertIsNotNone(
                self.monitoring_tree._create_frame_func(Frame))
            self.monitoring_tree.abs_path = "/tests/"
            _get_filename.return_value = "/tests/test_stats.py"
            self.assertIsNotNone(
                self.monitoring_tree._create_frame_func(Frame))

    def test__build_tree(self):
        class Code:
            co_firstlineno = 0
            co_name = ""
            co_code = ""

        class Frame:
            f_code = Code
            file_path = ""
            children = []

        self.assertIsNotNone(self.monitoring_tree._build_tree([Frame, Frame]))

        with patch(
                'auklet.stats.MonitoringTree._create_frame_func') as \
                create_frame_func:
            create_frame_func.return_value = Frame
            self.monitoring_tree._build_tree([Frame])
        self.assertIsNotNone(self.monitoring_tree._build_tree([Frame, Frame]))

    def test_update_sample_count(self):
        self.assertTrue(self.monitoring_tree._update_sample_count(
            parent=None, new_parent=self.get_new_parent()))

        if sys.version_info < (3,):
            self.build_assert_raises(
                "class NewChild has no attribute 'children'", True)
            self.build_assert_raises(
                "Parent instance has no attribute 'children'", False)
        else:
            self.build_assert_raises(
                "type object 'NewChild' has no attribute 'children'", True)
            self.build_assert_raises(
                "'Parent' object has no attribute 'children'", False)

    def test_update_hash(self):
        self.assertNotEqual(
            self.monitoring_tree.update_hash(new_stack=""), None)

        def _update_sample_count(self, parent, new_parent):
            global test_update_hash_parent  # used to tell if hash was created
            test_update_hash_parent = parent

        self.monitoring_tree.root_func = self.function
        with patch('auklet.stats.MonitoringTree._update_sample_count',
                   new=_update_sample_count):
            self.monitoring_tree.update_hash(new_stack="")

        self.assertNotEqual(test_update_hash_parent, None)  # global used here

    def test_clear_root(self):
        self.monitoring_tree.root_func = self.get_root_function()
        self.assertTrue(self.monitoring_tree.clear_root())
        self.assertEqual(self.monitoring_tree.root_func, None)

    def test_build_tree(self):
        self.monitoring_tree.root_func = None
        self.assertEqual({}, self.monitoring_tree.build_tree(self.Client()))
        self.monitoring_tree.root_func = self.get_root_function()
        self.assertIsNotNone(self.monitoring_tree.build_tree(self.Client()))

    def test_build_msgpack_tree(self):
        self.monitoring_tree.root_func = self.get_root_function()
        self.assertNotEqual(
            self.monitoring_tree.build_msgpack_tree(self.Client()),
            None)

    def get_code(self):
        class Code:
            co_code = "file_name"
            co_firstlineno = 0
            co_name = ""
        return Code

    def get_frame(self):
        class Frame:
            f_code = self.get_code()
        frame = [Frame, Frame]
        return frame

    def get_root_function(self):
        return {'callees': [],
                'filePath': None,
                'functionName': 'root',
                'lineNumber': 1,
                'nCalls': 1,
                'nSamples': 1}

    def get_parent(self):
        pass

    def get_new_parent(self):
        class Parent:
            children = []
        return Parent

    def build_assert_raises(self, expected, child_state):
        global child_exists  # For some reason this needs to be done twice
        child_exists = child_state
        with self.assertRaises(AttributeError) as error:
            self.monitoring_tree._update_sample_count(
                parent=self.Parent(), new_parent=self.NewParent())
        self.assertEqual(expected, str(error.exception))

    class Client:
        broker_username = "None"
        abs_path = "/Test/abs/path/"
        app_id = "12345"
        version = None

    class Parent:
        calls = 0
        samples = 0

        def has_child(self, new_child):
            global child_exists  # Must be used to pass variable in class
            if child_exists:
                class Child:
                    calls = 0
                    samples = 0

                child = Child
                child_exists = False
                return child
            else:
                return False

    class NewParent:
        class NewChild:
            calls = 0

        children = [NewChild]


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
import random
import string
from src.benchmark.statprof import statprof
from pidigits import piGenerator

__all__ = ['start']


class Fibonacci:
    def test(self, fibonacci_range=28):
        if fibonacci_range <= 1:
            return fibonacci_range
        else:
            return self.test(fibonacci_range-1) + self.test(fibonacci_range-2)


class PiDigits:
    @staticmethod
    def test(number_of_digits=30000):
        my_pi = piGenerator()
        return [next(my_pi) for _ in range(number_of_digits)]


class Addition:
    @staticmethod
    def test(number_of_iterations=10000000):
        total = 0
        for i in range(1, number_of_iterations):
            total = total + i


class Multiplication:
    @staticmethod
    def test(number_of_iterations=80000):
        total = 1
        for i in range(1, number_of_iterations):
            total = total * i


class Division:
    @staticmethod
    def test(number_of_iterations=1000000):
        total = 1
        for i in range(1, number_of_iterations):
            total = total / i


class WriteToDisk:
    @staticmethod
    def test():
        with open("/tmp/write-read", "w") as file:
            file.write(''.join(random.SystemRandom().choice(
                string.ascii_letters + string.digits) for _ in range(50000)))


class ReadFromDisk:
    @staticmethod
    def test():
        with open("/tmp/write-read", "r") as file:
            _ = file.read()


def runtest(state, object):
    statprof.start()
    test_name = object.__class__.__name__
    try:
        print("\nStarting %s tests..." % test_name)
        print("Running %s tests..." % test_name)
        object.test()
    finally:
        statprof.stop()
        print("%s test results..." % test_name)
        display(state, test_name)


def display(state, test_name):
    with open("/tmp/benchmark_results", "a") as file:
        file.write(state + " " + test_name + " ")
    statprof.display()
    statprof.reset()


def start(state):
    tests = [Fibonacci, PiDigits, Addition,
             Multiplication, Division, WriteToDisk, ReadFromDisk]

    for test in tests:
        runtest(state, test())




import json
import msgpack
import traceback

from time import time
from uuid import uuid4
from datetime import datetime
from auklet.stats import Event, SystemMetrics
from auklet.utils import create_file, \
    get_abs_path, get_device_ip, open_auklet_url, build_url, \
    get_agent_version, post_auklet_url, u

try:
    # For Python 3.0 and later
    from urllib.error import HTTPError, URLError
    from urllib.request import Request, urlopen
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import urlopen, Request, HTTPError, URLError


MB_TO_B = 1e6
S_TO_MS = 1000


class Client(object):
    producer_types = None
    brokers = None
    commit_hash = None
    mac_hash = None
    offline_filename = "{}/local.txt"
    limits_filename = "{}/limits"
    usage_filename = "{}/usage"
    com_config_filename = "{}/communication"
    identification_filename = "{}/identification"
    abs_path = None

    org_id = None
    client_id = None
    broker_username = None
    broker_password = None

    reset_data = False
    data_day = 1
    data_limit = None
    data_current = 0
    offline_limit = None
    offline_current = 0

    system_metrics = None

    def __init__(self, api_key=None, app_id=None, release=None,
                 base_url="https://api.auklet.io/", mac_hash=None,
                 version="", auklet_dir=""):
        self.apikey = api_key
        self.app_id = app_id
        self.base_url = base_url
        self.send_enabled = True
        self.producer = None
        self.mac_hash = mac_hash
        self.version = version
        self.auklet_dir = auklet_dir
        self._set_filenames()
        self._load_limits()
        create_file(self.offline_filename)
        create_file(self.limits_filename)
        create_file(self.usage_filename)
        create_file(self.com_config_filename)
        create_file(self.identification_filename)
        self.commit_hash = release
        self.abs_path = get_abs_path(".auklet/version")
        self.system_metrics = SystemMetrics()
        self._register_device()

    def _set_filenames(self):
        self.offline_filename = self.offline_filename.format(self.auklet_dir)
        self.limits_filename = self.limits_filename.format(self.auklet_dir)
        self.usage_filename = self.usage_filename.format(self.auklet_dir)
        self.com_config_filename = self.com_config_filename.format(
            self.auklet_dir)
        self.identification_filename = self.identification_filename.format(
            self.auklet_dir)

    def _register_device(self):
        try:
            read_id = json.loads(
                open(self.identification_filename, "r").read())
            if not read_id:
                raise IOError
            res, created = self.check_device(read_id['id'])
            if created:
                read_id = res
        except (IOError, ValueError):
            read_id = self.create_device()
        self.broker_password = read_id['client_password']
        self.broker_username = read_id['id']
        self.client_id = read_id['client_id']
        self.org_id = read_id['organization']
        self._write_identification({"id": self.broker_username,
                                    "client_password": self.broker_password,
                                    "organization": self.org_id,
                                    "client_id": self.client_id})
        return True

    def check_device(self, device_id):
        try:
            opened = open_auklet_url(
                build_url(
                    self.base_url,
                    "private/devices/{}/".format(device_id)
                ),
                self.apikey
            )
            res = json.loads(u(opened.read()))
            created = False
        except HTTPError:
            res = self.create_device()
            created = True
        return res, created

    def create_device(self):
        return post_auklet_url(
            build_url(
                self.base_url,
                "private/devices/"
            ),
            self.apikey,
            {"mac_address_hash": self.mac_hash, "application": self.app_id}
        )

    def _write_identification(self, data):
        with open(self.identification_filename, "w") as id_file:
            id_file.write(json.dumps(data))

    def _get_config(self):
        res = open_auklet_url(
            build_url(
                self.base_url,
                "private/devices/{}/app_config/".format(self.app_id)),
            self.apikey)
        if res is not None:
            return json.loads(u(res.read()))['config']

    def _load_limits(self):
        try:
            with open(self.limits_filename, "r") as limits:
                limits_str = limits.read()
                if limits_str:
                    data = json.loads(limits_str)
                    self.data_day = data['data']['normalized_cell_plan_date']
                    temp_limit = data['data']['cellular_data_limit']
                    if temp_limit is not None:
                        self.data_limit = data['data'][
                                              'cellular_data_limit'] * MB_TO_B
                    else:
                        self.data_limit = temp_limit
                    temp_offline = data['storage']['storage_limit']
                    if temp_offline is not None:
                        self.offline_limit = data['storage'][
                                                 'storage_limit'] * MB_TO_B
                    else:
                        self.offline_limit = data['storage']['storage_limit']
        except IOError:
            return

    def _build_usage_json(self):
        return {"data": self.data_current, "offline": self.offline_current}

    def _update_usage_file(self):
        try:
            with open(self.usage_filename, 'w') as usage:
                usage.write(json.dumps(self._build_usage_json()))
        except IOError:
            return False

    def check_data_limit(self, data, current_use, offline=False):
        if self.offline_limit is None and offline:
            return True
        if self.data_limit is None and not offline:
            return True
        data_size = len(data)
        temp_current = current_use + data_size
        if temp_current >= self.data_limit:
            return False
        if offline:
            self.offline_current = temp_current
        else:
            self.data_current = temp_current
        self._update_usage_file()
        return True

    def check_date(self):
        if datetime.today().day == self.data_day:
            if self.reset_data:
                self.data_current = 0
                self.reset_data = False
        else:
            self.reset_data = True

    def update_limits(self):
        config = self._get_config()
        if config is None:
            return 60000
        with open(self.limits_filename, 'w+') as limits:
            limits.truncate()
            limits.write(json.dumps(config))
        new_day = config['data']['normalized_cell_plan_date']
        temp_limit = config['data']['cellular_data_limit']
        if temp_limit is not None:
            new_data = config['data']['cellular_data_limit'] * MB_TO_B
        else:
            new_data = temp_limit
        temp_offline = config['storage']['storage_limit']
        if temp_offline is not None:
            new_offline = config['storage']['storage_limit'] * MB_TO_B
        else:
            new_offline = config['storage']['storage_limit']
        if self.data_day != new_day:
            self.data_day = new_day
            self.data_current = 0
        if self.data_limit != new_data:
            self.data_limit = new_data
        if self.offline_limit != new_offline:
            self.offline_limit = new_offline
        # return emission period in ms
        return config['emission_period'] * S_TO_MS

    def build_event_data(self, type, tb, tree):
        event = Event(type, tb, tree, self.abs_path)
        event_dict = dict(event)
        event_dict['application'] = self.app_id
        event_dict['publicIP'] = get_device_ip()
        event_dict['id'] = str(uuid4())
        event_dict['timestamp'] = int(round(time() * 1000))
        event_dict['systemMetrics'] = dict(self.system_metrics)
        event_dict['macAddressHash'] = self.mac_hash
        event_dict['release'] = self.commit_hash
        event_dict['agentVersion'] = get_agent_version()
        event_dict['device'] = self.broker_username
        event_dict['absPath'] = self.abs_path
        event_dict['version'] = self.version
        return event_dict

    def build_log_data(self, msg, data_type, level):
        log_dict = {
            "message": msg,
            "type": data_type,
            "level": level,
            "application": self.app_id,
            "publicIP": get_device_ip(),
            "id": str(uuid4()),
            "timestamp": int(round(time() * 1000)),
            "systemMetrics": dict(self.system_metrics),
            "macAddressHash": self.mac_hash,
            "release": self.commit_hash,
            "agentVersion": get_agent_version(),
            "device": self.broker_username,
            "version": self.version
        }
        return log_dict

    def build_msgpack_event_data(self, type, tb, tree):
        event_data = self.build_event_data(type, tb, tree)
        return msgpack.packb(event_data, use_bin_type=False)

    def build_msgpack_log_data(self, msg, data_type, level):
        log_data = self.build_log_data(msg, data_type, level)
        return msgpack.packb(log_data, use_bin_type=False)



import os
import ast
import unittest

from mock import patch
from tests import data_factory

from auklet.monitoring.processing import Client
from auklet.broker import MQTTClient


def recreate_files():
    os.system("touch .auklet/version")
    os.system("touch .auklet/communication")
    os.system("touch .auklet/usage")
    os.system("touch .auklet/limits")


class TestMQTTBroker(unittest.TestCase):
    data = ast.literal_eval(str(data_factory.MonitoringDataFactory()))
    config = ast.literal_eval(str(data_factory.ConfigFactory()))

    def setUp(self):
        with patch("auklet.monitoring.processing.Client._register_device",
                   new=self.__register_device):
            self.client = Client(
                api_key="", app_id="",
                base_url="https://api-staging.auklet.io/",
                auklet_dir=".auklet"
            )
            with patch('auklet.broker.MQTTClient._get_conf') as _get_conf:
                with patch("os.path.isfile") as is_file_mock:
                    is_file_mock.return_value = False
                    _get_conf.side_effect = self.get_conf
                    self.broker = MQTTClient(self.client)

    def test_write_conf(self):
        self.broker._write_conf(self.config)
        self.assertGreater(os.path.getsize(self.client.com_config_filename), 0)
        open(self.client.com_config_filename, "w").close()

    def test_get_certs(self):
        class urlopen:
            @staticmethod
            def read():
                with open("key.pem.zip", "rb") as file:
                    return file.read()

        self.assertFalse(self.broker._get_certs())
        with patch('auklet.broker.urlopen') as _urlopen:
            _urlopen.return_value = urlopen
            self.assertTrue(self.broker._get_certs())

    def test_read_from_conf(self):
        self.broker._read_from_conf({"brokers": "mqtt",
                                     "port": "8333",
                                     "prof_topic": "",
                                     "event_topic": "",
                                     "log_topic": ""})
        self.assertIsNotNone(self.broker.brokers)
        self.assertIsNotNone(self.broker.port)
        self.assertIsNotNone(self.broker.producer_types)

    def test_on_disconnect(self):
        def debug(msg):
            global debug_msg
            debug_msg = msg

        with patch('logging.debug') as _debug:
            _debug.side_effect = debug
            self.broker.on_disconnect(None, "", 1)
            self.assertIsNotNone(debug_msg)

    def test_create_producer(self):
        global create_producer_pass
        create_producer_pass = False

        with patch('auklet.broker.MQTTClient._get_certs') as get_certs:
            with patch('paho.mqtt.client.Client') as _Client:
                _Client.side_effect = self.MockClient
                get_certs.return_value = True
                os.system("touch .auklet/ck_ca.pem")
                self.broker.create_producer()
                self.assertTrue(create_producer_pass)

    def test_produce(self):
        with patch('paho.mqtt.client.Client.publish') as _publish:
            with patch('auklet.broker.MQTTClient._get_certs') as get_certs:
                with patch('paho.mqtt.client.Client') as _MQTT_Client:
                    _MQTT_Client.side_effect = self.MockClient
                    get_certs.return_value = True
                    _publish.side_effect = self.publish
                    self.broker.create_producer()
                    self.broker.produce(str(self.data))
                    self.assertIsNotNone(test_produce_payload)

    def test_get_conf(self):
        with patch('auklet.broker.open_auklet_url', new=self._open_auklet_url):
            with patch('auklet.broker.json.loads', new=self._loads):
                self.broker._get_conf()

    class MockClient:
        def __init__(self, client_id, protocol, transport, clean_session):
            pass

        def tls_set(self, ca_certs):
            pass

        def tls_set_context(self):
            pass

        def connect_async(self, broker, port):
            pass

        def enable_logger(self):
            pass

        def username_pw_set(self, username, password):
            pass

        def publish(self, topic, payload, qos=1):
            global test_produce_payload
            test_produce_payload = payload

        def loop_start(self):
            global create_producer_pass
            create_producer_pass = True

    @staticmethod
    def publish(data_type, payload):
        global test_produce_payload
        test_produce_payload = payload

    @staticmethod
    def get_conf():
        return True

    @staticmethod
    def _open_auklet_url(url, apikey):
        class MyObject:
            def read(self):
                return b"test_str"
        return MyObject()

    @staticmethod
    def _loads(data):
        return {
            "brokers": "mqtt",
            "port": "8883"
        }

    @staticmethod
    def __register_device(self):
        return True


if __name__ == '__main__':
    unittest.main()



import os
import ast
import json
import unittest

from mock import patch
from datetime import datetime

from tests import data_factory

from auklet.utils import b
from auklet.stats import MonitoringTree
from auklet.monitoring.processing import Client

try:
    # For Python 3.0 and later
    from urllib.error import HTTPError, URLError
    from urllib.request import Request, urlopen
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import urlopen, Request, HTTPError, URLError


class TestClient(unittest.TestCase):
    data = ast.literal_eval(str(data_factory.MonitoringDataFactory()))

    def setUp(self):
        with patch("auklet.monitoring.processing.Client._register_device",
                   new=self.__register_device):
            self.broker_username = "test-username"
            self.broker_password = "test-password"
            self.client = Client(
                api_key="", app_id="",
                base_url="https://api-staging.auklet.io/",
                auklet_dir=".auklet"
            )
            self.monitoring_tree = MonitoringTree()

    def test_get_config(self):
        class MockRequest:
            def __init__(self, *args, **kwargs):
                pass

            def get_type(self):
                pass

        class res:
            def read(self):
                return str(json.dumps({"config": ""})).encode()

        with patch('auklet.utils.urlopen') as _urlopen:
            with patch('auklet.utils.Request') as _Request:
                    _Request.side_effect = MockRequest
                    _urlopen.return_value = res()
                    self.assertIsNotNone(self.client._get_config())

    def test_register_device(self):
        with patch("json.loads", return_value=False):
            with patch("auklet.monitoring.processing.Client."
                       "create_device") as create_mock:
                create_mock.return_value = {
                    "client_password": "test-pass",
                    "client_id": "12345",
                    "id": "12345",
                    "organization": "12345"
                }
                self.assertTrue(self.client._register_device())

        with patch("json.loads") as loads_mock:
                with patch("auklet.monitoring.processing.Client."
                           "check_device") as check_mock:
                    loads_mock.return_value = {"id": "12345"}
                    check_mock.return_value = ({
                        "client_password": "test-pass",
                        "client_id": "12345",
                        "id": "12345",
                        "organization": "12345"
                    }, True)
                    self.assertTrue(self.client._register_device())

    def test_check_device(self):
        with patch("auklet.monitoring.processing.open_auklet_url",
                   return_value=self.MockResult()):
            res = self.client.check_device("123")
            self.assertFalse(res[1])
        with patch("auklet.monitoring.processing.open_auklet_url",
                   side_effect=HTTPError("url", 404, "failed post",
                                         {"test": "header"}, None)):
            res = self.client.check_device("123")
            self.assertTrue(res[1])

    def test_create_device(self):
        with patch("auklet.monitoring.processing.post_auklet_url",
                   return_value=self.MockResult()):
            res = self.client.create_device()
            self.assertEqual(res.read(), b(json.dumps({"test": "object"})))

    def test_load_limits(self):
        default_data = data_factory.LimitsGenerator()
        self.write_load_limits_test(default_data)
        self.build_load_limits_test(1, self.client.data_day)

        self.assertEqual(None, self.client.data_limit)
        self.assertEqual(None, self.client.offline_limit)

        data = data_factory.LimitsGenerator(cellular_data_limit=10)
        self.write_load_limits_test(data)
        self.client._load_limits()
        self.build_load_limits_test(10000000.0, self.client.data_limit)

        data = data_factory.LimitsGenerator(
            cellular_data_limit="null", storage_limit=10)
        self.write_load_limits_test(data)
        self.client._load_limits()
        self.build_load_limits_test(10000000.0, self.client.offline_limit)

        with open(self.client.limits_filename, "w") as limits:
            limits.write(str(default_data))

        self.base_patch_side_effect_with_none(
            'auklet.monitoring.processing.open',
            IOError, self.client._load_limits)

    def test_build_usage_json(self):
        data = self.client._build_usage_json()
        for value in data.values():
            self.assertNotEqual(value, None)

    def test_update_usage_file(self):
        os.system("rm -R .auklet")
        self.assertFalse(self.client._update_usage_file())
        os.system("mkdir .auklet")
        os.system("touch %s" % self.client.offline_filename)
        os.system("touch .auklet/version")

    def test_check_data_limit(self):
        self.client.offline_limit = None
        self.assertTrue(
            self.client.check_data_limit(
                data=self.data, current_use=0, offline=True))

        self.client.data_limit = None
        self.assertTrue(
            self.client.check_data_limit(data=self.data, current_use=0))

        self.client.offline_limit = self.client.data_limit = 1
        self.assertFalse(
            self.client.check_data_limit(data=self.data, current_use=0))

        self.client.data_limit = 1000
        self.client.check_data_limit(
            data=self.data, current_use=0, offline=True)
        self.assertNotEqual(self.client.offline_current, 0)

        self.client.check_data_limit(
            data=self.data, current_use=0)
        self.assertNotEqual(self.client.data_current, 0)

        self.assertTrue(
            self.client.check_data_limit(data=self.data, current_use=0))

    def test_check_date(self):
        self.client.data_day = datetime.today().day
        self.client.data_current = 1000
        self.client.reset_data = True
        self.client.check_date()
        self.assertEqual(self.client.data_current, 0)
        self.assertEqual(self.client.reset_data, False)
        self.client.data_day = datetime.today().day+1
        self.client.check_date()
        self.assertTrue(self.client.reset_data)

    def test_update_limits(self):
        with patch(
                'auklet.monitoring.processing.Client._get_config',
                new=self._get_config):
            self.assertEqual(self.client.update_limits(), 60000)
            self.none = False

            self.cellular_data_limit = 1000
            self.client.update_limits()
            self.assertEqual(self.client.data_limit, 1000000000.0)
            self.cellular_data_limit = None

            self.storage_limit = 1000
            self.client.update_limits()
            self.assertEqual(self.client.offline_limit, 1000000000.0)
            self.storage_limit = None

            self.cell_plan_date = 10
            self.client.update_limits()
            self.assertEqual(self.client.data_day, 10)
            self.cell_plan_date = 1

            self.assertEqual(self.client.update_limits(), 60000)

    def test_build_event_data(self):
        with patch("auklet.monitoring.processing.traceback.format_exc",
                   new=self.mock_format_exc):
            self.assertBuildEventData(self.client.build_event_data)

    def test_build_log_data(self):
        self.assertBuildLogData(
            self.client.build_log_data(
                msg='msg', data_type='data_type', level='level'))

    def test_build_msgpack_event_data(self):
        with patch("auklet.monitoring.processing.traceback.format_exc",
                   new=self.mock_format_exc):
            self.assertBuildEventData(self.client.build_msgpack_event_data)

    def test_build_msgpack_log_data(self):
        self.assertBuildLogData(
            self.client.build_msgpack_log_data(
                msg='msg', data_type='data_type', level='level'))

    def mock_format_exc(self):
        return ""

    @staticmethod
    def get_mock_event(exc_type=None, tb=None, tree=None, abs_path=None):
        return {"stackTrace":
                [{"functionName": "",
                  "filePath": "",
                  "lineNumber": 0,
                  "locals":
                    {"key": "value"}}]}

    def base_patch_side_effect_with_none(self, location, side_effect, actual):
        with patch(location) as _base:
            _base.side_effect = side_effect
            self.assertIsNone(actual())

    def build_load_limits_test(self, expected, actual):
        self.assertEqual(expected, actual)

    def write_load_limits_test(self, data):
        filename = self.client.limits_filename
        with open(filename, "w") as limits:
            limits.write(str(data))

    def assertBuildEventData(self, function):
        def traceback():
            class Code:
                co_code = "file_name"
                co_name = ""

            class Frame:
                f_code = Code()
                f_lineno = 0
                f_locals = ""

            class Traceback:
                tb_lineno = 0
                tb_frame = Frame()
                tb_next = None
            return Traceback

        with patch(
                'auklet.monitoring.processing.Event', new=self.get_mock_event):
            self.monitoring_tree.cached_filenames["file_name"] = "file_name"
            self.assertNotEqual(
                function(
                    type=Exception,
                    tb=traceback(),
                    tree=self.monitoring_tree),
                None)

    def assertBuildLogData(self, function):
        self.assertNotEqual(function, None)

    none = True
    cellular_data_limit = None
    storage_limit = None
    cell_plan_date = 1

    def _get_config(self):
        if self.none:
            return None
        else:
            return {"storage":
                        {"storage_limit": self.storage_limit},
                    "emission_period": 60,
                    "features":
                        {"performance_metrics": True,
                         "user_metrics": False},
                    "data":
                        {"cellular_data_limit": self.cellular_data_limit,
                         "normalized_cell_plan_date": self.cell_plan_date}}

    @staticmethod
    def __register_device(self):
        return True

    class MockResult(object):
        def read(self):
            return b(json.dumps({"test": "object"}))



if __name__ == '__main__':
    unittest.main()



import factory


class MonitoringDataGenerator(object):
    def __init__(self, data):
        self.commitHash = data[0]
        self.id = data[1]
        self.publicIP = data[2]
        self.timestamp = data[3]
        self.application = data[4]
        self.macAddressHash = data[5]

        self.lineNumber = data[6]
        self.nSamples = data[7]
        self.functionName = data[8]
        self.nCalls = data[9]

    def __str__(self):
        return '{"commitHash": "%s", ' \
               '"id": "%s", ' \
               '"publicIP": "%s", ' \
               '"timestamp": %d, ' \
               '"application": "%s", ' \
               '"macAddressHash": "%s", ' \
               '"callees": ' \
               '[{"lineNumber": %d, ' \
               '"nSamples": %d, ' \
               '"functionName": "%s", ' \
               '"nCalls": %d}]}' \
               % (self.commitHash, self.id, self.publicIP,
                  self.timestamp, self.application, self.macAddressHash,
                  self.lineNumber, self.nSamples, self.functionName,
                  self.nCalls)


class MonitoringDataFactory(factory.Factory):
    class Meta:
        model = MonitoringDataGenerator

    data = list()

    data.append("d0eb7082f4dd5dfce4c543e21299fb2e5774f70b")
    data.append("30d376d2-fc7e-10d5-d51fe33373fd")
    data.append("187.2.167.60")
    data.append(int(1531490785464))
    data.append("nugvjtNBxHbjBnqbcFZvjn")
    data.append("d1dd34825af8599b78bd5f4a1d7d186e")
    data.append(int(836))
    data.append(int(5745))
    data.append("root")
    data.append(0)


class ConfigGenerator(object):
    def __init__(self, data):
        self.brokers = data[0]
        self.prof_topic = data[1]
        self.event_topic = data[2]
        self.log_topic = data[3]
        self.user_metrics_topic = data[4]

    def __str__(self):
        return '{"brokers": ["%s"], "prof_topic": "%s", "event_topic": ' \
               '"%s", "log_topic": "%s", "user_metrics_topic": "%s"}' \
               % (self.brokers, self.prof_topic, self.event_topic,
                  self.log_topic, self.user_metrics_topic)


class ConfigFactory(factory.Factory):
    class Meta:
        model = ConfigGenerator

    data = list()

    data.append("http://api-staging.auklet.io:9093")
    data.append("profiling")
    data.append("events")
    data.append("logging")
    data.append("user_metrics")


class StackTraceGenerator(object):
    def __init__(self, data):
        self.functionName = data[0]
        self.lineNumber = data[1]
        self.nCalls = data[2]
        self.nSamples = data[3]

    def __str__(self):
        return '''{'callees': [],\n 'filePath': None,\n ''' \
               ''''functionName': '%s',\n 'lineNumber': %d,\n ''' \
               ''''nCalls': %d,\n 'nSamples': %d}''' \
               % (self.functionName, self.lineNumber,
                  self.nCalls, self.nSamples)


class StackTraceFactory(factory.Factory):
    class Meta:
        model = StackTraceGenerator

    data = list()

    data.append("root")
    data.append(1)
    data.append(1)
    data.append(1)


class SingleNestedStackGenerator(object):
    def __init__(self, data):
        self.callees_functionName = data[0]
        self.callees_lineNumber = data[1]
        self.callees_nCalls = data[2]
        self.callees_nSamples = data[3]
        self.functionName = data[4]
        self.lineNumber = data[5]
        self.nCalls = data[6]
        self.nSamples = data[7]

    def __str__(self):
        return '''{'callees': [{'callees': [],\n''' \
               '''              'filePath': None,\n''' \
               '''              'functionName': '%s',\n''' \
               '''              'lineNumber': %d,\n''' \
               '''              'nCalls': %d,\n''' \
               '''              'nSamples': %d}],\n 'filePath': None,\n''' \
               ''' 'functionName': '%s',\n 'lineNumber': %d,\n''' \
               ''' 'nCalls': %d,\n 'nSamples': %d}''' \
               % (self.callees_functionName, self.callees_lineNumber,
                  self.callees_nCalls, self.callees_nSamples,
                  self.functionName, self.lineNumber, self.nCalls,
                  self.nSamples)


class SingleNestedStackTraceFactory(factory.Factory):
    class Meta:
        model = SingleNestedStackGenerator

    data = list()

    data.append("")
    data.append(0)
    data.append(1)
    data.append(1)

    data.append("root")
    data.append(1)
    data.append(1)
    data.append(1)


class LimitsGenerator(object):
    def __init__(self, storage_limit="null", emission_period=60,
                 performance_metrics="true", user_metrics="false",
                 cellular_data_limit="null", normalized_cell_plan_date=1):
        self.storage_limit = str(storage_limit)
        self.emission_period = str(emission_period)
        self.performance_metrics = str(performance_metrics)
        self.user_metrics = str(user_metrics)
        self.cellular_data_limit = str(cellular_data_limit)
        self.normalized_cell_plan_data = str(normalized_cell_plan_date)

    def __str__(self):
        return """{"storage": {"storage_limit": """ + \
               self.storage_limit + \
               """}, "emission_period": """ + \
               self.emission_period + \
               """, "features": {"performance_metrics": """ + \
               self.performance_metrics + \
               """, "user_metrics": """ + \
               self.user_metrics + \
               """}, "data": {"cellular_data_limit": """ + \
               self.cellular_data_limit + \
               """, "normalized_cell_plan_date": """ + \
               self.normalized_cell_plan_data + \
               """}}"""



import unittest

from auklet.monitoring.logging import AukletLogging


class TestAukletLogging(unittest.TestCase):
    def setUp(self):
        self.auklet_logging = AukletLogging()

    def base_test_log(self, function):
        self.assertRaises(
            NotImplementedError, lambda: function(
                msg="", data_type=str))

    def test_log(self):
        self.base_test_log(self.auklet_logging.log)

    def test_debug(self):
        self.base_test_log(self.auklet_logging.debug)

    def test_info(self):
        self.base_test_log(self.auklet_logging.info)

    def test_warning(self):
        self.base_test_log(self.auklet_logging.warning)

    def test_error(self):
        self.base_test_log(self.auklet_logging.error)

    def test_critical(self):
        self.base_test_log(self.auklet_logging.critical)


if __name__ == '__main__':
    unittest.main()



import os
import sys
import uuid
import hashlib
import requests
import tempfile

from auklet.__about__ import __version__ as auklet_version
from auklet.errors import AukletConfigurationError

__all__ = ['open_auklet_url', 'post_auklet_url', 'create_file', 'clear_file',
           'build_url', 'get_mac', 'get_device_ip', 'create_dir',
           'setup_thread_excepthook', 'get_abs_path', 'get_agent_version',
           'b', 'u']

try:
    # For Python 3.0 and later
    from urllib.error import HTTPError, URLError
    from urllib.request import Request, urlopen
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import urlopen, Request, HTTPError, URLError


def open_auklet_url(url, apikey):
    url = Request(url, headers={"Authorization": "JWT %s" % apikey})
    try:
        res = urlopen(url)
    except HTTPError as e:
        if e.code == 401:
            raise AukletConfigurationError(
                "Invalid configuration of Auklet Monitoring, "
                "ensure proper API key and app ID passed to "
                "Monitoring class"
            )
        raise e
    except URLError:
        return None
    return res


def post_auklet_url(url, apikey, data):
    try:
        res = requests.post(
            url,
            json=data,
            headers={"Authorization": "JWT %s" % apikey,
                     "Content-Type": "application/json"})
    except requests.HTTPError:
        return None
    return res.json()


def create_file(filename):
    open(filename, "a").close()


def create_dir(dir_name=".auklet"):
    dirs = [os.getcwd(), os.path.expanduser("~")]
    for directory in dirs:
        full_path = "{}/{}".format(directory, dir_name)
        if os.access(directory, os.W_OK):
            if not os.path.exists(full_path):
                os.mkdir(full_path)
            return full_path
    return tempfile.gettempdir()


def clear_file(filename):
    open(filename, "w").close()


def build_url(base_url, extension):
    return '{}{}'.format(base_url, extension)


def get_mac():
    mac_num = hex(uuid.getnode()).replace('0x', '').upper()
    mac = '-'.join(mac_num[i: i + 2] for i in range(0, 11, 2))
    return hashlib.md5(b(mac)).hexdigest()


def get_abs_path(path):
    try:
        return os.path.abspath(path).split('/.auklet')[0]
    except IndexError:
        return ''


def get_device_ip():
    try:
        request = Request("https://api.ipify.org")
        res = urlopen(request)
        return u(res.read())
    except (HTTPError, URLError, Exception):
        # TODO log to kafka if the ip service fails for any reason
        return None


def get_agent_version():
    return auklet_version


def setup_thread_excepthook():
    import threading
    """
    Workaround for `sys.excepthook` thread bug from:
    http://bugs.python.org/issue1230540

    Call once from the main thread before creating any threads.
    """
    init_original = threading.Thread.__init__

    def init(self, *args, **kwargs):

        init_original(self, *args, **kwargs)
        run_original = self.run

        def run_with_except_hook(*args2, **kwargs2):
            try:
                run_original(*args2, **kwargs2)
            except (KeyboardInterrupt, SystemExit):
                raise
            except:
                sys.excepthook(*sys.exc_info())

        self.run = run_with_except_hook

    threading.Thread.__init__ = init


if sys.version_info < (3,):
    # Python 2 and 3 String Compatibility
    def b(x):
        return x

    def u(x):
        return x
else:
    # https://pythonhosted.org/six/#binary-and-text-data
    import codecs

    def b(x):
        # Produces a unicode string to encoded bytes
        return codecs.utf_8_encode(x)[0]

    def u(x):
        # Produces a byte string from a unicode object
        return codecs.utf_8_decode(x)[0]
